@@ -29,7 +29,7 @@
 #
 #########################################################################
 #
-# $Id: install.pl,v 1.215 2004/10/25 04:40:47 mbr Exp $
+# $Id: install.pl,v 1.223 2005/03/12 17:13:44 mbr Exp $
 #
 
 use Cwd;
@@ -81,6 +81,38 @@ my $iptablesCmd  = '/sbin/iptables';
 my $psadCmd      = "${USRSBIN_DIR}/psad";
 #============ end config ============
 
+### map perl modules to versions
+my %required_perl_modules = (
+    'Unix::Syslog' => {
+        'version' => '0.100',
+        'force-lib-install' => 0,
+    },
+    'Bit::Vector' => {
+        'version' => '6.3',
+        'force-lib-install' => 0,
+    },
+    'Date::Calc', => {
+        'version' => '5.3',
+        'force-lib-install' => 0,
+    },
+    'Net::IPv4Addr' => {
+        'version' => '0.10',
+        'force-lib-install' => 0,
+    },
+    'IPTables::Parse' => {
+        'version' => '0.2',
+        'force-lib-install' => 1,
+    },
+    'IPTables::ChainMgr' => {
+        'version' => '0.1',
+        'force-lib-install' => 1,
+    },
+    'Psad' => {
+        'version' => '1.4.1',
+        'force-lib-install' => 1,
+    }
+);
+
 ### IP regex
 my $ip_re = '(?:\d{1,3}\.){3}\d{1,3}';
 
@@ -117,15 +149,17 @@ unless ($found) {
     $PERL_INSTALL_DIR = $INC[0];
 }
 
-### set the default execution flags
-my $noarchive    = 0;
-my $uninstall    = 0;
-my $help         = 0;
+### set the default execution flags and command line args
+my $noarchive   = 0;
+my $uninstall   = 0;
+my $help        = 0;
+my $syslog_conf = '';
 
 &usage(1) unless (GetOptions(
-    'no-preserve' => \$noarchive,   ### Don't preserve existing configs.
-    'uninstall'   => \$uninstall,   ### Uninstall psad.
-    'help'        => \$help         ### Display help.
+    'no-preserve'   => \$noarchive,   ### Don't preserve existing configs.
+    'syslog-conf=s' => \$syslog_conf, ### specify path to syslog config file.
+    'uninstall'     => \$uninstall,   ### Uninstall psad.
+    'help'          => \$help         ### Display help.
 ));
 &usage(0) if $help;
 
@@ -192,7 +226,7 @@ exit 0;
 
 sub install() {
     ### make sure install.pl is being called from the source directory
-    unless (-e 'psad' && -e 'Psad/Psad.pm') {
+    unless (-e 'psad' && -e 'Psad/lib/Psad.pm') {
         die "[*] install.pl can only be executed from the directory\n",
             "    that contains the psad sources!  Exiting.";
     }
@@ -211,10 +245,6 @@ sub install() {
         &logr("[+] Creating $VARLIBDIR\n");
         mkdir $VARLIBDIR, 0500;
     }
-    unless (-d $LIBDIR) {
-        &logr("[+] Creating $LIBDIR\n");
-        mkdir $LIBDIR, 0755;
-    }
 
     ### deal with old psad_auto_ips path
     if (-e "${PSAD_CONFDIR}/psad_auto_ips") {
@@ -266,30 +296,6 @@ sub install() {
         }
     }
 
-    my $restarted_syslog = 0;
-    if (-e '/etc/syslog.conf') {
-        &append_fifo_syslog();
-        if (((system "$Cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
-            &logr("[+] Restarted syslog.\n");
-            $restarted_syslog = 1;
-        }
-    }
-    if (-e '/etc/syslog-ng/syslog-ng.conf') {
-        &append_fifo_syslog_ng();
-        if (((system "$Cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
-            &logr("[+] Restarted syslog-ng.\n");
-            $restarted_syslog = 1;
-        }
-    }
-    if (-e '/etc/metalog/metalog.conf') {
-        &config_metalog();
-        &logr("[-] Metalog support is shaky in psad.  " .
-            "Use at your own risk.\n");
-    }
-
-    unless ($restarted_syslog) {
-        &logr("[-] Could not restart any syslog daemons.\n");
-    }
     unless (-d $PSAD_DIR) {
         &logr("[+] Creating $PSAD_DIR\n");
         mkdir $PSAD_DIR, 0500;
@@ -322,14 +328,11 @@ sub install() {
     print "\n\n";
 
     ### install perl modules
-    &install_perl_module('Unix::Syslog', 'Unix-Syslog', '0.100');
-    &install_perl_module('Bit::Vector', 'Bit-Vector', '6.3');
-    &install_perl_module('Date::Calc', 'Date-Calc', '5.3');
-    &install_perl_module('Net::IPv4Addr', 'Net-IPv4Addr', '0.10');
-    &install_perl_module('IPTables::Parse', 'IPTables/Parse', '0.10');
-    &install_perl_module('Psad', 'Psad', '1.3.3');
+    for my $module (keys %required_perl_modules) {
+        &install_perl_module($module);
+    }
 
-    &logr("[+] Installing snort-2.1 signatures in $SNORT_DIR\n");
+    &logr("[+] Installing Snort-2.3 signatures in $SNORT_DIR\n");
     unless (-d $SNORT_DIR) {
         mkdir $SNORT_DIR, 0500 or die "[*] Could not create $SNORT_DIR: $!";
     }
@@ -427,6 +430,10 @@ sub install() {
         &logr("[+] Creating $PSAD_CONFDIR\n");
         mkdir $PSAD_CONFDIR,0500;
     }
+
+    ### get syslog daemon (e.g. syslog, syslog-ng, or metalog)
+    my $syslog_str = &query_syslog();
+
     my $preserve_rv = 0;
     if (-e "${PSAD_CONFDIR}/psad.conf") {
         $preserve_rv = &query_preserve_config();
@@ -435,7 +442,7 @@ sub install() {
     ### the order of the config files is important (legacy FW_MSG_SEARCH
     ### vars in psad.conf).
     for my $file qw(psad.conf psadwatchd.conf
-            kmsgsd.conf fw_search.conf) {
+            kmsgsd.conf fw_search.conf alert.conf) {
         if (-e "${PSAD_CONFDIR}/$file") {
             &archive("${PSAD_CONFDIR}/$file") unless $noarchive;
             if ($preserve_rv) {
@@ -492,13 +499,6 @@ sub install() {
     }
     &logr("\n");
 
-    if (-x $Cmds{'iptables'}) {
-        &logr("[+] Found iptables.  Testing syslog configuration:\n");
-        ### make sure we actually see packets being logged by
-        ### the firewall.
-        &test_syslog_config();
-    }
-
     unless ($preserve_rv) {  ### we want to preserve the existing config
 
         ### get email address(es)
@@ -511,13 +511,6 @@ sub install() {
             }
         }
 
-        ### get syslog daemon (e.g. syslog, syslog-ng, or metalog)
-        my $syslog_str = &query_syslog();
-        if ($syslog_str) {
-            &put_string('SYSLOG_DAEMON', $syslog_str,
-                "${PSAD_CONFDIR}/psad.conf");
-        }
-
         ### Give the admin the opportunity to set the strings that are
         ### parsed out of iptables messages.  This is useful since the
         ### admin may have configured the firewall to use a logging prefix
@@ -568,6 +561,47 @@ sub install() {
         }
     }
 
+    &put_string('SYSLOG_DAEMON', $syslog_str,
+        "${PSAD_CONFDIR}/psad.conf");
+
+    my $restarted_syslog = 0;
+    if ($syslog_str eq 'syslogd') {
+        if (-e $syslog_conf) {
+            &append_fifo_syslog($syslog_conf);
+            if (((system "$Cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
+                &logr("[+] HUP signal sent to syslogd.\n");
+                $restarted_syslog = 1;
+            }
+        }
+    } elsif ($syslog_str eq 'syslog-ng') {
+        if (-e $syslog_conf) {
+            &append_fifo_syslog_ng($syslog_conf);
+            if (((system "$Cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
+                &logr("[+] HUP signal sent to syslog-ng.\n");
+                $restarted_syslog = 1;
+            }
+        }
+    } elsif ($syslog_str eq 'metalog') {
+        if (-e $syslog_conf) {
+            &config_metalog($syslog_conf);
+            &logr("[-] Metalog support is shaky in psad.  " .
+                "Use at your own risk.\n");
+            ### don't send warning about not restarting metalog daemon
+            $restarted_syslog = 1;
+        }
+    }
+
+    unless ($restarted_syslog) {
+        &logr("[-] Could not restart any syslog daemons.\n");
+    }
+
+    if (-x $Cmds{'iptables'}) {
+        &logr("[+] Found iptables. Testing syslog configuration:\n");
+        ### make sure we actually see packets being logged by
+        ### the firewall.
+        &test_syslog_config($syslog_str);
+    }
+
     ### see if there are any "_CHANGEME_" strings left and give the
     ### admin a chance to correct (this can happen if a new config
     ### variable is introduced in a new version of psad but the
@@ -822,28 +856,59 @@ sub stop_psad() {
 }
 
 sub install_perl_module() {
-    my ($mod_name, $mod_dir, $version) = @_;
-    &logr("[+] Installing the $mod_name $version perl module\n");
-    chdir $mod_dir or die "[*] Could not chdir to ",
-        "$mod_dir: $!";
-    unless (-e 'Makefile.PL') {
-        die "[*] Your $mod_name source directory appears to be incomplete!\n",
-            "    Download the latest sources from ",
-            "http://www.cipherdyne.org\n";
-    }
-    system "$Cmds{'make'} clean" if -e 'Makefile';
-    system "$Cmds{'perl'} Makefile.PL PREFIX=$LIBDIR LIB=$LIBDIR";
-    system $Cmds{'make'};
-#    system "$Cmds{'make'} test";
-    system "$Cmds{'make'} install";
-    chdir $src_dir;
+    my $mod_name = shift;
 
-    print "\n\n";
+    die '[*] Missing version key in required_perl_modules hash.'
+        unless defined $required_perl_modules{$mod_name}{'version'};
+    die '[*] Missing force-lib-install key in required_perl_modules hash.'
+        unless defined $required_perl_modules{$mod_name}{'force-lib-install'};
+
+    my $version = $required_perl_modules{$mod_name}{'version'};
+
+    my $install_module = 0;
+
+    if ($required_perl_modules{$mod_name}{'force-lib-install'}) {
+        ### install regardless of whether the module may already be
+        ### installed
+        $install_module = 1;
+    } elsif (not has_perl_module($mod_name)) {
+        ### install the module in the /usr/lib/psad directory because
+        ### it is not already installed.
+        $install_module = 1;
+    }
+
+    if ($install_module) {
+        unless (-d $LIBDIR) {
+            &logr("[+] Creating $LIBDIR\n");
+            mkdir $LIBDIR, 0755 or die "[*] Could not mkdir $LIBDIR: $!";
+        }
+        &logr("[+] Installing the $mod_name $version perl module\n");
+        my $mod_dir = $mod_name;
+        $mod_dir =~ s/::/-/g;
+        chdir $mod_dir or die "[*] Could not chdir to ",
+            "$mod_dir: $!";
+        unless (-e 'Makefile.PL') {
+            die "[*] Your $mod_name source directory appears to be incomplete!\n",
+                "    Download the latest sources from ",
+                "http://www.cipherdyne.org\n";
+        }
+        system "$Cmds{'make'} clean" if -e 'Makefile';
+        system "$Cmds{'perl'} Makefile.PL PREFIX=$LIBDIR LIB=$LIBDIR";
+        system $Cmds{'make'};
+#        system "$Cmds{'make'} test";
+        system "$Cmds{'make'} install";
+        chdir $src_dir;
+
+        print "\n\n";
+    }
     return;
 }
 
 sub set_home_net() {
     my $file = shift;
+
+    ### get all interfaces; even those that are down since they may
+    ### brought up any time.
     my @ifconfig_out = `$Cmds{'ifconfig'} -a`;
     my $home_net_str = '';
     my $intf_name = '';
@@ -864,7 +929,25 @@ sub set_home_net() {
         }
         if ($intf_name and
                 $line =~ /^\s+inet\s+.*?:($ip_re).*:($ip_re)/) {
-            $connected_subnets{$intf_name} = "$1/$2";
+            my $ip = $1;
+            my $mask = $2;
+
+            my @ipbytes = split /\./, $ip;
+            my @mbytes  = split /\./, $mask;
+
+            my $netaddr = '';
+            for (my $i=0; $i < 4; $i++) {
+                my $byte1 = $mbytes[$i]+0;
+                my $byte2 = $ipbytes[$i]+0;
+                my $netaddr_byte = $byte1 & $byte2;
+                if ($i != 0) {
+                    $netaddr = $netaddr . '.' . $netaddr_byte;
+                } else {
+                    $netaddr = $netaddr_byte;
+                }
+            }
+
+            $connected_subnets{$intf_name} = "$netaddr/$mask";
             $net_ctr++;
         }
     }
@@ -945,18 +1028,9 @@ sub set_hostname() {
 }
 
 sub append_fifo_syslog_ng() {
-    &logr('[+] Modifying /etc/syslog-ng/syslog-ng.conf to write kern.info ' .
-        "messages to\n");
-    &logr("    $PSAD_FIFO\n");
-    unless (-e '/etc/syslog-ng/syslog-ng.conf.orig') {
-        copy '/etc/syslog-ng/syslog-ng.conf',
-            '/etc/syslog-ng/syslog-ng.conf.orig' or die "[*] Could not ",
-            "copy /etc/syslog-ng/syslog-ng.conf -> ",
-            "/etc/syslog-ng/syslog-ng.conf.orig: $!";
-    }
-    &archive('/etc/syslog-ng/syslog-ng.conf');
-    open RS, '< /etc/syslog-ng/syslog-ng.conf' or
-        die "[*] Unable to open /etc/syslog-ng/syslog-ng.conf: $!\n";
+    my $syslog_conf = shift;
+    open RS, "< $syslog_conf" or
+        die "[*] Unable to open $syslog_conf: $!\n";
     my @slines = <RS>;
     close RS;
 
@@ -966,8 +1040,16 @@ sub append_fifo_syslog_ng() {
     }
 
     unless ($found_fifo) {
-        open SYSLOGNG, '>> /etc/syslog-ng/syslog-ng.conf' or
-            die "[*] Unable to open /etc/syslog-ng/syslog-ng.conf: $!\n";
+        &logr("[+] Modifying $syslog_conf to write kern.info " .
+            "messages to\n    $PSAD_FIFO");
+        unless (-e "$syslog_conf.orig") {
+            copy $syslog_conf, "$syslog_conf.orig" or die "[*] Could not ",
+                "copy $syslog_conf -> $syslog_conf.orig: $!";
+        }
+        &archive($syslog_conf);
+
+        open SYSLOGNG, ">> $syslog_conf" or
+            die "[*] Unable to open $syslog_conf: $!\n";
         print SYSLOGNG "\n",
             "destination psadpipe { pipe(\"/var/lib/psad/psadfifo\"); };\n",
 #            "filter f_kerninfo { facility(kern) and level(info); };\n",
@@ -979,13 +1061,9 @@ sub append_fifo_syslog_ng() {
 }
 
 sub config_metalog() {
-    unless (-e '/etc/metalog/metalog.conf.orig') {
-        copy '/etc/metalog/metalog.conf',
-            '/etc/metalog/metalog.conf.orig' or die "[*] Could not copy ",
-            "/etc/metalog/metalog.conf -> /etc/metalog/metalog.conf.orig: $!";
-    }
-    open RS, "< /etc/metalog/metalog.conf" or
-        die "[*] Unable to open /etc/metalog/metalog.conf: $!\n";
+    my $syslog_conf = shift;
+    open RS, "< $syslog_conf" or
+        die "[*] Unable to open $syslog_conf: $!\n";
     my @lines = <RS>;
     close RS;
 
@@ -997,8 +1075,14 @@ sub config_metalog() {
         }
     }
     unless ($found) {
-        open METALOG, "> /etc/metalog/metalog.conf" or
-            die "[*] Unable to open /etc/metalog/metalog.conf: $!";
+        &logr("[+] Modifying $syslog_conf to write kern.info messages " .
+            "to\n    $PSAD_FIFO (with script /usr/sbin/psadpipe.sh)");
+        unless (-e "$syslog_conf.orig") {
+            copy $syslog_conf, "$syslog_conf.orig" or die "[*] Could not copy ",
+                "$syslog_conf -> $syslog_conf.orig: $!";
+        }
+        open METALOG, "> $syslog_conf" or
+            die "[*] Unable to open $syslog_conf: $!";
 
         print METALOG "\n";
         print METALOG "\nPSAD :\n",
@@ -1028,7 +1112,7 @@ sub config_metalog() {
 sub query_preserve_config() {
     my $ans = '';
     while ($ans ne 'y' && $ans ne 'n') {
-        &logr('[+] Would you like to preserve the config from the ' .
+        &logr('[+] Would you like to merge the config from the ' .
             "existing\n");
         &logr("    psad installation ([y]/n)?  ");
         $ans = <STDIN>;
@@ -1046,7 +1130,7 @@ sub query_preserve_sigs_autodl() {
     my $ans = '';
     while ($ans ne 'y' && $ans ne 'n') {
         &logr("\n");
-        &logr("[+] Preserve any user modfications in $file ([y]/n)?  ");
+        &logr("[+] Merge any user modfications in $file ([y]/n)?  ");
         $ans = <STDIN>;
         $ans = 'y' if $ans eq "\n";
         chomp $ans;
@@ -1149,34 +1233,44 @@ sub preserve_config() {
 }
 
 sub append_fifo_syslog() {
-    &logr('[+] Modifying /etc/syslog.conf to write kern.info ' .
-        "messages to\n");
-    &logr("    $PSAD_FIFO\n");
-    unless (-e '/etc/syslog.conf.orig') {
-        copy '/etc/syslog.conf', '/etc/syslog.conf.orig' or die "[*] Could ",
-            "not copy /etc/syslog.conf -> /etc/syslog.conf.orig: $!";
-    }
-    &archive('/etc/syslog.conf');
-    open RS, '< /etc/syslog.conf' or
-        die "[*] Unable to open /etc/syslog.conf: $!\n";
+    my $syslog_conf = shift;
+    open RS, "< $syslog_conf" or
+        die "[*] Unable to open $syslog_conf: $!\n";
     my @slines = <RS>;
     close RS;
-    open SYSLOG, '> /etc/syslog.conf' or
-        die "[*] Unable to open /etc/syslog.conf: $!\n";
+
+    my $found_fifo = 0;
     for my $line (@slines) {
-        unless ($line =~ /psadfifo/) {
-            print SYSLOG $line;
+        $found_fifo = 1 if $line =~ /psadfifo/;
+    }
+
+    unless ($found_fifo) {
+        &logr("[+] Modifying $syslog_conf to write kern.info " .
+            "messages to\n");
+        &logr("    $PSAD_FIFO\n");
+        unless (-e "$syslog_conf.orig") {
+            copy $syslog_conf, "$syslog_conf.orig" or die "[*] Could ",
+                "not copy $syslog_conf -> $syslog_conf.orig: $!";
         }
+        &archive($syslog_conf);
+        open SYSLOG, "> $syslog_conf" or
+            die "[*] Unable to open $syslog_conf: $!\n";
+        for my $line (@slines) {
+            unless ($line =~ /psadfifo/) {
+                print SYSLOG $line;
+            }
+        }
+        print SYSLOG '### Send kern.info messages to psadfifo for ',
+            "analysis by kmsgsd\n";
+        ### reinstate kernel logging to our named pipe
+        print SYSLOG "kern.info\t\t|$PSAD_FIFO\n";
+        close SYSLOG;
     }
-    print SYSLOG '### Send kern.info messages to psadfifo for ',
-        "analysis by kmsgsd\n";
-    ### reinstate kernel logging to our named pipe
-    print SYSLOG "kern.info\t\t|$PSAD_FIFO\n";
-    close SYSLOG;
     return;
 }
 
 sub test_syslog_config() {
+    my $syslog_str = shift;
     my %used_ports;
 
     ### first find an unused high tcp port to use for testing
@@ -1292,11 +1386,11 @@ sub test_syslog_config() {
     &scrub_fwdata();
 
     if ($found) {
-        &logr("[+] Successful syslog reconfiguration.\n\n");
+        &logr("[+] Successful $syslog_str reconfiguration.\n\n");
     } else {
-        &logr("[-] unsuccessful syslog reconfiguration.\n");
+        &logr("[-] unsuccessful $syslog_str reconfiguration.\n");
         &logr("    Consult the psad man page for the basic " .
-            "syslog requirement to get psad to work.\n\n");
+            "$syslog_str requirement to get psad to work.\n\n");
     }
 
     if ($start_kmsgsd && -e "${RUNDIR}/kmsgsd.pid") {
@@ -1375,13 +1469,7 @@ sub check_old_psad_installation() {
     unlink "${PERL_INSTALL_DIR}/Psad.pm"
         if (-e "${PERL_INSTALL_DIR}/Psad.pm");
     if (-e '/var/log/psadfifo') {  ### this is the old psadfifo location
-        if (-e "${USRSBIN_DIR}/psad"
-            && system "${USRSBIN_DIR}/psad --Status > /dev/null") {
-            ### deal with this later.  The user should be prompted before
-            ### the old psadfifo is removed since kmsgsd will have a problem
-        } else {
-            unlink '/var/log/psadfifo';
-        }
+        unlink '/var/log/psadfifo';
     }
     return;
 }
@@ -1565,8 +1653,30 @@ sub query_syslog() {
         &logr("    Which system logger is running (syslogd, " .
             "syslog-ng, metalog)?  ");
         $ans = <STDIN>;
-        chomp $ans;
+        $ans =~ s/\s*//g;
+
+        if ($ans eq 'syslogd') {
+            ### allow command line --syslog-conf arg to take over
+            $syslog_conf = '/etc/syslog.conf' unless $syslog_conf;
+        } elsif ($ans eq 'syslog-ng') {
+            ### allow command line --syslog-conf arg to take over
+            $syslog_conf = '/etc/syslog-ng/syslog-ng.conf' unless $syslog_conf;
+        } elsif ($ans eq 'metalog') {
+            ### allow command line --syslog-conf arg to take over
+            $syslog_conf = '/etc/metalog/metalog.conf' unless $syslog_conf;
+        }
+        if ($syslog_conf and not -e $syslog_conf) {
+            die
+"[-] The config file $syslog_conf does not exist. Re-run install.pl\n",
+"    with the --syslog-conf argument to specify the path to the syslog\n",
+"    daemon config file.\n";
+        }
     }
+    die "[-] Invalid syslog daemon \"$ans\"\n"
+        unless ($ans and
+            ($ans eq 'syslogd'
+            or $ans eq 'syslog-ng'
+            or $ans eq 'metalog'));
     print "\n";
     return $ans;
 }
@@ -1765,6 +1875,18 @@ sub install_manpage() {
     return;
 }
 
+sub has_perl_module() {
+    my $module = shift;
+
+    # 5.8.0 has a bug with require Foo::Bar alone in an eval, so an
+    # extra statement is a workaround.
+    my $file = "$module.pm";
+    $file =~ s{::}{/}g;
+    eval { require $file };
+
+    return $@ ? 0 : 1;
+}
+
 ### logging subroutine that handles multiple filehandles
 sub logr() {
     my $msg = shift;
