@@ -29,7 +29,7 @@
 *     USA
 ******************************************************************************
 *
-*  $Id: kmsgsd.c,v 1.46 2004/11/26 03:57:16 mbr Exp $
+*  $Id: kmsgsd.c,v 1.47 2005/03/08 05:50:37 mbr Exp $
 */
 
 /* includes */
@@ -188,7 +188,7 @@ int main(int argc, char *argv[]) {
 
             if ((fwdata_fd = open(fwdata_file, O_CREAT|O_WRONLY|O_APPEND,
                     0600)) < 0) {
-                fprintf(stderr, "** Could not open %s for writing.\n",
+                fprintf(stderr, "[*] Could not open %s for writing.\n",
                     fwdata_file);
                 exit(EXIT_FAILURE);  /* could not open fwdata file */
             }
